from .syntax.parser cimport Parser
from .syntax.beam_parser cimport BeamParser
from .syntax.ner cimport BiluoPushDown
from .syntax.arc_eager cimport ArcEager
from .tagger cimport Tagger


cdef class EntityRecognizer(Parser):
    pass


cdef class DependencyParser(Parser):
    pass


cdef class BeamEntityRecognizer(BeamParser):
    pass


cdef class BeamDependencyParser(BeamParser):
    pass



# coding: utf-8
from __future__ import unicode_literals
from __future__ import division

from ...serialize.huffman import HuffmanCodec
from ...serialize.bits import BitArray


from heapq import heappush, heappop, heapify
from collections import defaultdict
import numpy
import pytest


def py_encode(symb2freq):
    """Huffman encode the given dict mapping symbols to weights
    From Rosetta Code
    """
    heap = [[wt, [sym, ""]] for sym, wt in symb2freq.items()]
    heapify(heap)
    while len(heap) > 1:
        lo = heappop(heap)
        hi = heappop(heap)
        for pair in lo[1:]:
            pair[1] = '0' + pair[1]
        for pair in hi[1:]:
            pair[1] = '1' + pair[1]
        heappush(heap, [lo[0] + hi[0]] + lo[1:] + hi[1:])
    return dict(heappop(heap)[1:])


def test_serialize_huffman_1():
    probs = numpy.zeros(shape=(10,), dtype=numpy.float32)
    probs[0] = 0.3
    probs[1] = 0.2
    probs[2] = 0.15
    probs[3] = 0.1
    probs[4] = 0.06
    probs[5] = 0.02
    probs[6] = 0.01
    probs[7] = 0.005
    probs[8] = 0.0001
    probs[9] = 0.000001

    codec = HuffmanCodec(list(enumerate(probs)))
    py_codes = py_encode(dict(enumerate(probs)))
    py_codes = list(py_codes.items())
    py_codes.sort()
    assert codec.strings == [c for i, c in py_codes]


def test_serialize_huffman_empty():
    codec = HuffmanCodec({})
    assert codec.strings == []


def test_serialize_huffman_round_trip():
    words = ['the', 'quick', 'brown', 'fox', 'jumped', 'over', 'the', 'the',
             'lazy', 'dog', '.']
    freqs = {'the': 10, 'quick': 3, 'brown': 4, 'fox': 1, 'jumped': 5,
             'over': 8, 'lazy': 1, 'dog': 2, '.': 9}

    codec = HuffmanCodec(freqs.items())
    strings = list(codec.strings)
    codes = dict([(codec.leaves[i], strings[i]) for i in range(len(codec.leaves))])
    bits = codec.encode(words)
    string = ''.join('{0:b}'.format(c).rjust(8, '0')[::-1] for c in bits.as_bytes())
    for word in words:
        code = codes[word]
        assert string[:len(code)] == code
        string = string[len(code):]
    unpacked = [0] * len(words)
    bits.seek(0)
    codec.decode(bits, unpacked)
    assert words == unpacked


def test_serialize_huffman_rosetta():
    text = "this is an example for huffman encoding"
    symb2freq = defaultdict(int)
    for ch in text:
        symb2freq[ch] += 1
    by_freq = list(symb2freq.items())
    by_freq.sort(reverse=True, key=lambda item: item[1])
    symbols = [sym for sym, prob in by_freq]

    codec = HuffmanCodec(symb2freq.items())
    py_codec = py_encode(symb2freq)

    codes = dict([(codec.leaves[i], codec.strings[i]) for i in range(len(codec.leaves))])

    my_lengths = defaultdict(int)
    py_lengths = defaultdict(int)
    for symb, freq in symb2freq.items():
        my = codes[symb]
        my_lengths[len(my)] += freq
        py_lengths[len(py_codec[symb])] += freq
    my_exp_len = sum(length * weight for length, weight in my_lengths.items())
    py_exp_len = sum(length * weight for length, weight in py_lengths.items())
    assert my_exp_len == py_exp_len


@pytest.mark.models
def test_vocab(EN):
    codec = HuffmanCodec([(w.orth, numpy.exp(w.prob)) for w in EN.vocab])
    expected_length = 0
    for i, code in enumerate(codec.strings):
        leaf = codec.leaves[i]
        expected_length += len(code) * numpy.exp(EN.vocab[leaf].prob)
    assert 8 < expected_length < 15



# coding: utf8
from __future__ import unicode_literals

import regex as re

from spacy.language_data.punctuation import ALPHA_LOWER, CURRENCY
from ..language_data.tokenizer_exceptions import _URL_PATTERN

ABBREVIATIONS = """
A.
AG.
AkH.
Aö.
B.
B.CS.
B.S.
B.Sc.
B.ú.é.k.
BE.
BEK.
BSC.
BSc.
BTK.
Bat.
Be.
Bek.
Bfok.
Bk.
Bp.
Bros.
Bt.
Btk.
Btke.
Btét.
C.
CSC.
Cal.
Cg.
Cgf.
Cgt.
Cia.
Co.
Colo.
Comp.
Copr.
Corp.
Cos.
Cs.
Csc.
Csop.
Cstv.
Ctv.
Ctvr.
D.
DR.
Dipl.
Dr.
Dsz.
Dzs.
E.
EK.
EU.
F.
Fla.
Folyt.
Fpk.
Főszerk.
G.
GK.
GM.
Gfv.
Gmk.
Gr.
Group.
Gt.
Gy.
H.
HKsz.
Hmvh.
I.
Ifj.
Inc.
Inform.
Int.
J.
Jr.
Jv.
K.
K.m.f.
KB.
KER.
KFT.
KRT.
Kb.
Ker.
Kft.
Kg.
Kht.
Kkt.
Kong.
Korm.
Kr.
Kr.e.
Kr.u.
Krt.
L.
LB.
Llc.
Ltd.
M.
M.A.
M.S.
M.SC.
M.Sc.
MA.
MH.
MSC.
MSc.
Mass.
Max.
Mlle.
Mme.
Mo.
Mr.
Mrs.
Ms.
Mt.
N.
N.N.
NB.
NBr.
Nat.
No.
Nr.
Ny.
Nyh.
Nyr.
Nyrt.
O.
OJ.
Op.
P.
P.H.
P.S.
PH.D.
PHD.
PROF.
Pf.
Ph.D
PhD.
Pk.
Pl.
Plc.
Pp.
Proc.
Prof.
Ptk.
R.
RT.
Rer.
Rt.
S.
S.B.
SZOLG.
Salg.
Sch.
Spa.
St.
Sz.
SzRt.
Szerk.
Szfv.
Szjt.
Szolg.
Szt.
Sztv.
Szvt.
Számv.
T.
TEL.
Tel.
Ty.
Tyr.
U.
Ui.
Ut.
V.
VB.
Vcs.
Vhr.
Vht.
Várm.
W.
X.
X.Y.
Y.
Z.
Zrt.
Zs.
a.C.
ac.
adj.
adm.
ag.
agit.
alez.
alk.
all.
altbgy.
an.
ang.
arch.
at.
atc.
aug.
b.a.
b.s.
b.sc.
bek.
belker.
berend.
biz.
bizt.
bo.
bp.
br.
bsc.
bt.
btk.
ca.
cc.
cca.
cf.
cif.
co.
corp.
cos.
cs.
csc.
csüt.
cső.
ctv.
dbj.
dd.
ddr.
de.
dec.
dikt.
dipl.
dj.
dk.
dl.
dny.
dolg.
dr.
du.
dzs.
ea.
ed.
eff.
egyh.
ell.
elv.
elvt.
em.
eng.
eny.
et.
etc.
ev.
ezr.
eü.
f.h.
f.é.
fam.
fb.
febr.
fej.
felv.
felügy.
ff.
ffi.
fhdgy.
fil.
fiz.
fm.
foglalk.
ford.
fp.
fr.
frsz.
fszla.
fszt.
ft.
fuv.
főig.
főisk.
főtörm.
főv.
gazd.
gimn.
gk.
gkv.
gmk.
gondn.
gr.
grav.
gy.
gyak.
gyártm.
gör.
hads.
hallg.
hdm.
hdp.
hds.
hg.
hiv.
hk.
hm.
ho.
honv.
hp.
hr.
hrsz.
hsz.
ht.
htb.
hv.
hőm.
i.e.
i.sz.
id.
ie.
ifj.
ig.
igh.
ill.
imp.
inc.
ind.
inform.
inic.
int.
io.
ip.
ir.
irod.
irod.
isk.
ism.
izr.
iá.
jan.
jav.
jegyz.
jgmk.
jjv.
jkv.
jogh.
jogt.
jr.
jvb.
júl.
jún.
karb.
kat.
kath.
kb.
kcs.
kd.
ker.
kf.
kft.
kht.
kir.
kirend.
kisip.
kiv.
kk.
kkt.
klin.
km.
korm.
kp.
krt.
kt.
ktsg.
kult.
kv.
kve.
képv.
kísérl.
kóth.
könyvt.
körz.
köv.
közj.
közl.
közp.
közt.
kü.
lat.
ld.
legs.
lg.
lgv.
loc.
lt.
ltd.
ltp.
luth.
m.a.
m.s.
m.sc.
ma.
mat.
max.
mb.
med.
megh.
met.
mf.
mfszt.
min.
miss.
mjr.
mjv.
mk.
mlle.
mme.
mn.
mozg.
mr.
mrs.
ms.
msc.
má.
máj.
márc.
mé.
mélt.
mü.
műh.
műsz.
műv.
művez.
nagyker.
nagys.
nat.
nb.
neg.
nk.
no.
nov.
nu.
ny.
nyilv.
nyrt.
nyug.
obj.
okl.
okt.
old.
olv.
orsz.
ort.
ov.
ovh.
pf.
pg.
ph.d
ph.d.
phd.
phil.
pjt.
pk.
pl.
plb.
plc.
pld.
plur.
pol.
polg.
poz.
pp.
proc.
prof.
prot.
pság.
ptk.
pu.
pü.
r.k.
rac.
rad.
red.
ref.
reg.
rer.
rev.
rf.
rkp.
rkt.
rt.
rtg.
röv.
s.b.
s.k.
sa.
sb.
sel.
sgt.
sm.
st.
stat.
stb.
strat.
stud.
sz.
szakm.
szaksz.
szakszerv.
szd.
szds.
szept.
szerk.
szf.
szimf.
szjt.
szkv.
szla.
szn.
szolg.
szt.
szubj.
szöv.
szül.
tanm.
tb.
tbk.
tc.
techn.
tek.
tel.
tf.
tgk.
ti.
tip.
tisztv.
titks.
tk.
tkp.
tny.
tp.
tszf.
tszk.
tszkv.
tv.
tvr.
ty.
törv.
tü.
ua.
ui.
unit.
uo.
uv.
vas.
vb.
vegy.
vh.
vhol.
vhr.
vill.
vizsg.
vk.
vkf.
vkny.
vm.
vol.
vs.
vsz.
vv.
vál.
várm.
vízv.
vö.
zrt.
zs.
Á.
Áe.
Áht.
É.
Épt.
Ész.
Új-Z.
ÚjZ.
Ún.
á.
ált.
ápr.
ásv.
é.
ék.
ény.
érk.
évf.
í.
ó.
össz.
ötk.
özv.
ú.
ú.n.
úm.
ún.
út.
üag.
üd.
üdv.
üe.
ümk.
ütk.
üv.
ű.
őrgy.
őrpk.
őrv.
""".strip().split()

OTHER_EXC = """
-e
""".strip().split()

ORD_NUM_OR_DATE = "([A-Z0-9]+[./-])*(\d+\.?)"
_NUM = "[+\-]?\d+([,.]\d+)*"
_OPS = "[=<>+\-\*/^()÷%²]"
_SUFFIXES = "-[{a}]+".format(a=ALPHA_LOWER)
NUMERIC_EXP = "({n})(({o})({n}))*[%]?".format(n=_NUM, o=_OPS)
TIME_EXP = "\d+(:\d+)*(\.\d+)?"

NUMS = "(({ne})|({t})|({on})|({c}))({s})?".format(
    ne=NUMERIC_EXP, t=TIME_EXP, on=ORD_NUM_OR_DATE,
    c=CURRENCY, s=_SUFFIXES
)

TOKEN_MATCH = re.compile("^({u})|({n})$".format(u=_URL_PATTERN, n=NUMS)).match



# coding: utf8
from __future__ import unicode_literals


ADVERBS_IRREG = {
    "best": ("well",),
    "better": ("well",),
    "deeper": ("deeply",),
    "farther": ("far",),
    "further": ("far",),
    "harder": ("hard",),
    "hardest": ("hard",)
}



# coding: utf-8
from __future__ import unicode_literals

import pytest


@pytest.mark.parametrize('text,number', [("7am", "7"), ("11p.m.", "11")])
def test_issue736(en_tokenizer, text, number):
    """Test that times like "7am" are tokenized correctly and that numbers are converted to string."""
    tokens = en_tokenizer(text)
    assert len(tokens) == 2
    assert tokens[0].text == number



# coding: utf8
from __future__ import unicode_literals

from ..symbols import *
from ..language_data import PRON_LEMMA

# Source https://www.cs.tut.fi/~jkorpela/kielenopas/5.5.html

TOKENIZER_EXCEPTIONS = {
    "aik.": [
        {ORTH: "aik.", LEMMA: "aikaisempi"}
    ],
    "alk.": [
        {ORTH: "alk.", LEMMA: "alkaen"}
    ],
    "alv.": [
        {ORTH: "alv.", LEMMA: "arvonlisävero"}
    ],
    "ark.": [
        {ORTH: "ark.", LEMMA: "arkisin"}
    ],
    "as.": [
        {ORTH: "as.", LEMMA: "asunto"}
    ],
    "ed.": [
        {ORTH: "ed.", LEMMA: "edellinen"}
    ],
    "esim.": [
        {ORTH: "esim.", LEMMA: "esimerkki"}
    ],
    "huom.": [
        {ORTH: "huom.", LEMMA: "huomautus"}
    ],
    "jne.": [
        {ORTH: "jne.", LEMMA: "ja niin edelleen"}
    ],
    "joht.": [
        {ORTH: "joht.", LEMMA: "johtaja"}
    ],
    "k.": [
        {ORTH: "k.", LEMMA: "kuollut"}
    ],
    "ks.": [
        {ORTH: "ks.", LEMMA: "katso"}
    ],
    "lk.": [
        {ORTH: "lk.", LEMMA: "luokka"}
    ],
    "lkm.": [
        {ORTH: "lkm.", LEMMA: "lukumäärä"}
    ],
    "lyh.": [
        {ORTH: "lyh.", LEMMA: "lyhenne"}
    ],
    "läh.": [
        {ORTH: "läh.", LEMMA: "lähettäjä"}
    ],
    "miel.": [
        {ORTH: "miel.", LEMMA: "mieluummin"}
    ],
    "milj.": [
        {ORTH: "milj.", LEMMA: "miljoona"}
    ],
    "mm.": [
        {ORTH: "mm.", LEMMA: "muun muassa"}
    ],
    "myöh.": [
        {ORTH: "myöh.", LEMMA: "myöhempi"}
    ],
    "n.": [
        {ORTH: "n.", LEMMA: "noin"}
    ],
    "nimim.": [
        {ORTH: "nimim.", LEMMA: "nimimerkki"}
    ],
    "ns.": [
        {ORTH: "ns.", LEMMA: "niin sanottu"}
    ],
    "nyk.": [
        {ORTH: "nyk.", LEMMA: "nykyinen"}
    ],
    "oik.": [
        {ORTH: "oik.", LEMMA: "oikealla"}
    ],
    "os.": [
        {ORTH: "os.", LEMMA: "osoite"}
    ],
    "p.": [
        {ORTH: "p.", LEMMA: "päivä"}
    ],
    "par.": [
        {ORTH: "par.", LEMMA: "paremmin"}
    ],
    "per.": [
        {ORTH: "per.", LEMMA: "perustettu"}
    ],
    "pj.": [
        {ORTH: "pj.", LEMMA: "puheenjohtaja"}
    ],
    "puh.joht.": [
        {ORTH: "puh.joht.", LEMMA: "puheenjohtaja"}
    ],
    "prof.": [
        {ORTH: "prof.", LEMMA: "professori"}
    ],
    "puh.": [
        {ORTH: "puh.", LEMMA: "puhelin"}
    ],
    "pvm.": [
        {ORTH: "pvm.", LEMMA: "päivämäärä"}
    ],
    "rak.": [
        {ORTH: "rak.", LEMMA: "rakennettu"}
    ],
    "ry.": [
        {ORTH: "ry.", LEMMA: "rekisteröity yhdistys"}
    ],
    "s.": [
        {ORTH: "s.", LEMMA: "sivu"}
    ],
    "siht.": [
        {ORTH: "siht.", LEMMA: "sihteeri"}
    ],
    "synt.": [
        {ORTH: "synt.", LEMMA: "syntynyt"}
    ],
    "t.": [
        {ORTH: "t.", LEMMA: "toivoo"}
    ],
    "tark.": [
        {ORTH: "tark.", LEMMA: "tarkastanut"}
    ],
    "til.": [
        {ORTH: "til.", LEMMA: "tilattu"}
    ],
    "tms.": [
        {ORTH: "tms.", LEMMA: "tai muuta sellaista"}
    ],
    "toim.": [
        {ORTH: "toim.", LEMMA: "toimittanut"}
    ],
    "v.": [
        {ORTH: "v.", LEMMA: "vuosi"}
    ],
    "vas.": [
        {ORTH: "vas.", LEMMA: "vasen"}
    ],
    "vast.": [
        {ORTH: "vast.", LEMMA: "vastaus"}
    ],
    "vrt.": [
        {ORTH: "vrt.", LEMMA: "vertaa"}
    ],
    "yht.": [
        {ORTH: "yht.", LEMMA: "yhteensä"}
    ],
    "yl.": [
        {ORTH: "yl.", LEMMA: "yleinen"}
    ],
    "ym.": [
        {ORTH: "ym.", LEMMA: "ynnä muuta"}
    ],
    "yms.": [
        {ORTH: "yms.", LEMMA: "ynnä muuta sellaista"}
    ],
    "yo.": [
        {ORTH: "yo.", LEMMA: "ylioppilas"}
    ],
    "yliopp.": [
        {ORTH: "yliopp.", LEMMA: "ylioppilas"}
    ],
    "ao.": [
        {ORTH: "ao.", LEMMA: "asianomainen"}
    ],
    "em.": [
        {ORTH: "em.", LEMMA: "edellä mainittu"}
    ],
    "ko.": [
        {ORTH: "ko.", LEMMA: "kyseessä oleva"}
    ],
    "ml.": [
        {ORTH: "ml.", LEMMA: "mukaan luettuna"}
    ],
    "po.": [
        {ORTH: "po.", LEMMA: "puheena oleva"}
    ],
    "so.": [
        {ORTH: "so.", LEMMA: "se on"}
    ],
    "ts.": [
        {ORTH: "ts.", LEMMA: "toisin sanoen"}
    ],
    "vm.": [
        {ORTH: "vm.", LEMMA: "viimeksi mainittu"}
    ],
    "srk.": [
        {ORTH: "srk.", LEMMA: "seurakunta"}
    ]
}



# coding: utf8
from __future__ import unicode_literals, print_function

from .tokenizer_exceptions import TOKEN_MATCH
from .language_data import *
from ..attrs import LANG
from ..language import Language


class Hungarian(Language):
    lang = 'hu'

    class Defaults(Language.Defaults):
        tokenizer_exceptions = dict(TOKENIZER_EXCEPTIONS)
        lex_attr_getters = dict(Language.Defaults.lex_attr_getters)
        lex_attr_getters[LANG] = lambda text: 'hu'

        prefixes = tuple(TOKENIZER_PREFIXES)

        suffixes = tuple(TOKENIZER_SUFFIXES)

        infixes = tuple(TOKENIZER_INFIXES)

        stop_words = set(STOP_WORDS)

        token_match = TOKEN_MATCH



# coding: utf-8
from __future__ import unicode_literals

from ..util import get_doc

import pytest


@pytest.mark.models
def test_issue514(EN):
    """Test serializing after adding entity"""
    text = ["This", "is", "a", "sentence", "about", "pasta", "."]
    vocab = EN.entity.vocab
    doc = get_doc(vocab, text)
    EN.entity.add_label("Food")
    EN.entity(doc)
    label_id = vocab.strings[u'Food']
    doc.ents = [(label_id, 5,6)]
    assert [(ent.label_, ent.text) for ent in doc.ents] == [("Food", "pasta")]
    doc2 = get_doc(EN.entity.vocab).from_bytes(doc.to_bytes())
    assert [(ent.label_, ent.text) for ent in doc2.ents] == [("Food", "pasta")]



# coding: utf8
from __future__ import unicode_literals

from .doc import Doc
from ..symbols import HEAD, TAG, DEP, ENT_IOB, ENT_TYPE


def merge_ents(doc):
    """
    Helper: merge adjacent entities into single tokens; modifies the doc.
    """
    for ent in doc.ents:
        ent.merge(ent.root.tag_, ent.text, ent.label_)
    return doc


def format_POS(token, light, flat):
    """
    Helper: form the POS output for a token.
    """
    subtree = dict([
        ("word", token.text),
        ("lemma", token.lemma_),  # trigger
        ("NE", token.ent_type_),  # trigger
        ("POS_fine", token.tag_),
        ("POS_coarse", token.pos_),
        ("arc", token.dep_),
        ("modifiers", [])
    ])
    if light:
        subtree.pop("lemma")
        subtree.pop("NE")
    if flat:
        subtree.pop("arc")
        subtree.pop("modifiers")
    return subtree


def POS_tree(root, light=False, flat=False):
    """
    Helper: generate a POS tree for a root token. The doc must have
    merge_ents(doc) ran on it.
    """
    subtree = format_POS(root, light=light, flat=flat)
    for c in root.children:
        subtree["modifiers"].append(POS_tree(c))
    return subtree


def parse_tree(doc, light=False, flat=False):
    """
    Makes a copy of the doc, then construct a syntactic parse tree, similar to
    the one used in displaCy. Generates the POS tree for all sentences in a doc.

    Args:
        doc: The doc for parsing.

    Returns:
        [parse_trees (Dict)]:

    >>> from spacy.en import English
    >>> nlp = English()
    >>> doc = nlp('Bob brought Alice the pizza. Alice ate the pizza.')
    >>> trees = doc.print_tree()
    [{'modifiers': [{'modifiers': [], 'NE': 'PERSON', 'word': 'Bob', 'arc': 'nsubj', 'POS_coarse': 'PROPN', 'POS_fine': 'NNP', 'lemma': 'Bob'}, {'modifiers': [], 'NE': 'PERSON', 'word': 'Alice', 'arc': 'dobj', 'POS_coarse': 'PROPN', 'POS_fine': 'NNP', 'lemma': 'Alice'}, {'modifiers': [{'modifiers': [], 'NE': '', 'word': 'the', 'arc': 'det', 'POS_coarse': 'DET', 'POS_fine': 'DT', 'lemma': 'the'}], 'NE': '', 'word': 'pizza', 'arc': 'dobj', 'POS_coarse': 'NOUN', 'POS_fine': 'NN', 'lemma': 'pizza'}, {'modifiers': [], 'NE': '', 'word': '.', 'arc': 'punct', 'POS_coarse': 'PUNCT', 'POS_fine': '.', 'lemma': '.'}], 'NE': '', 'word': 'brought', 'arc': 'ROOT', 'POS_coarse': 'VERB', 'POS_fine': 'VBD', 'lemma': 'bring'}, {'modifiers': [{'modifiers': [], 'NE': 'PERSON', 'word': 'Alice', 'arc': 'nsubj', 'POS_coarse': 'PROPN', 'POS_fine': 'NNP', 'lemma': 'Alice'}, {'modifiers': [{'modifiers': [], 'NE': '', 'word': 'the', 'arc': 'det', 'POS_coarse': 'DET', 'POS_fine': 'DT', 'lemma': 'the'}], 'NE': '', 'word': 'pizza', 'arc': 'dobj', 'POS_coarse': 'NOUN', 'POS_fine': 'NN', 'lemma': 'pizza'}, {'modifiers': [], 'NE': '', 'word': '.', 'arc': 'punct', 'POS_coarse': 'PUNCT', 'POS_fine': '.', 'lemma': '.'}], 'NE': '', 'word': 'ate', 'arc': 'ROOT', 'POS_coarse': 'VERB', 'POS_fine': 'VBD', 'lemma': 'eat'}]
    """
    doc_clone = Doc(doc.vocab, words=[w.text for w in doc])
    doc_clone  = Doc(doc.vocab, words=[w.text for w in doc])
    doc_clone.from_array([HEAD, TAG, DEP, ENT_IOB, ENT_TYPE],
        doc.to_array([HEAD, TAG, DEP, ENT_IOB, ENT_TYPE]))
    merge_ents(doc_clone)  # merge the entities into single tokens first
    return [POS_tree(sent.root, light=light, flat=flat) for sent in doc_clone.sents]



# coding: utf8
from __future__ import unicode_literals

from spacy.language_data import strings_to_exc, update_exc
from .punctuation import *
from .stop_words import STOP_WORDS
from .tokenizer_exceptions import ABBREVIATIONS
from .tokenizer_exceptions import OTHER_EXC
from .. import language_data as base

STOP_WORDS = set(STOP_WORDS)

TOKENIZER_EXCEPTIONS = strings_to_exc(base.EMOTICONS)
update_exc(TOKENIZER_EXCEPTIONS, strings_to_exc(base.ABBREVIATIONS))
update_exc(TOKENIZER_EXCEPTIONS, strings_to_exc(OTHER_EXC))
update_exc(TOKENIZER_EXCEPTIONS, strings_to_exc(ABBREVIATIONS))

TOKENIZER_PREFIXES = TOKENIZER_PREFIXES
TOKENIZER_SUFFIXES = TOKENIZER_SUFFIXES
TOKENIZER_INFIXES = TOKENIZER_INFIXES

__all__ = ["TOKENIZER_EXCEPTIONS", "STOP_WORDS", "TOKENIZER_PREFIXES", "TOKENIZER_SUFFIXES", "TOKENIZER_INFIXES"]
